@@ -25,17 +25,16 @@
  * SECTION:element-cmmldec
  * @see_also: cmmlenc, oggdemux
  *
- * <refsect2>
- * <para>
  * Cmmldec extracts a CMML document from a CMML bitstream.<ulink
  * url="http://www.annodex.net/TR/draft-pfeiffer-cmml-02.html">CMML</ulink> is
  * an XML markup language for time-continuous data maintained by the <ulink
  * url="http:/www.annodex.org/">Annodex Foundation</ulink>.
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipeline</title>
- * <programlisting>
+ * |[
  * gst-launch -v filesrc location=annotated.ogg ! oggdemux ! cmmldec ! filesink location=annotations.cmml
- * </programlisting>
+ * ]|
  * </refsect2>
  */
 
@@ -533,7 +532,7 @@ gst_cmml_dec_parse_xml (GstCmmlDec * dec, guchar * data, guint size)
   GError *err = NULL;
 
   if (!gst_cmml_parser_parse_chunk (dec->parser, (gchar *) data, size, &err)) {
-    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL), (err->message));
+    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL), ("%s", err->message));
     g_error_free (err);
     dec->flow_return = GST_FLOW_ERROR;
   }
