@@ -48,9 +48,11 @@
 
 #include <gst/gst.h>
 #include <gst/gst-i18n-plugin.h>
+#include <gst/pbutils/pbutils.h>
 
 #include "gstapedemux.h"
 
+#include <stdio.h>
 #include <string.h>
 
 #define APE_VERSION_MAJOR(ver)  ((ver)/1000)
@@ -62,7 +64,6 @@ static const GstElementDetails gst_ape_demux_details =
 GST_ELEMENT_DETAILS ("APE tag demuxer",
     "Codec/Demuxer/Metadata",
     "Read and output APE tags while demuxing the contents",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>, "
     "Tim-Philipp Müller <tim centricular net>");
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -350,6 +351,7 @@ gst_ape_demux_parse_tag (GstTagDemux * demux, GstBuffer * buffer,
   const guint8 *footer;
   gboolean have_header;
   gboolean end_tag = !start_tag;
+  GstCaps *sink_caps;
   guint version, footer_size;
 
   GST_LOG_OBJECT (demux, "Parsing buffer of size %u", GST_BUFFER_SIZE (buffer));
@@ -409,6 +411,11 @@ gst_ape_demux_parse_tag (GstTagDemux * demux, GstBuffer * buffer,
 
   *tags = ape_demux_parse_tags (data, *tag_size - footer_size);
 
+  sink_caps = gst_static_pad_template_get_caps (&sink_factory);
+  gst_pb_utils_add_codec_description_to_tag_list (*tags,
+      GST_TAG_CONTAINER_FORMAT, sink_caps);
+  gst_caps_unref (sink_caps);
+
   return GST_TAG_DEMUX_RESULT_OK;
 }
 
