@@ -83,7 +83,7 @@ static const GstElementDetails gst_avi_mux_details =
 GST_ELEMENT_DETAILS ("Avi muxer",
     "Codec/Muxer",
     "Muxes audio and video into an avi stream",
-    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+    "GStreamer maintainers <gstreamer-devel@lists.sourceforge.net>");
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -150,7 +150,11 @@ static GstStaticPadTemplate video_sink_factory =
         "video/x-wmv, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], " "framerate = (fraction) [ 0, MAX ], "
-        "wmvversion = (int) [ 1, 3]")
+        "wmvversion = (int) [ 1, 3];"
+        "image/x-jpc, "
+        "width = (int) [ 1, 2147483647 ], "
+        "height = (int) [ 1, 2147483647 ], "
+        "framerate = (fraction) [ 0, MAX ]")
     );
 
 static GstStaticPadTemplate audio_sink_factory =
@@ -605,6 +609,8 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
             break;
         }
       }
+    } else if (!strcmp (mimetype, "image/x-jpc")) {
+      avipad->vids.compression = GST_MAKE_FOURCC ('M', 'J', '2', 'C');
     }
 
     if (!avipad->vids.compression)
@@ -630,6 +636,96 @@ refuse_caps:
   }
 }
 
+static GstFlowReturn
+gst_avi_mux_audsink_scan_mpeg_audio (GstAviMux * avimux, GstAviPad * avipad,
+    GstBuffer * buffer)
+{
+  guint8 *data;
+  guint size;
+  guint spf;
+  guint32 header;
+  gulong layer;
+  gulong version;
+  gint lsf, mpg25;
+
+  data = GST_BUFFER_DATA (buffer);
+  size = GST_BUFFER_SIZE (buffer);
+
+  if (size < 4)
+    goto not_parsed;
+
+  header = GST_READ_UINT32_BE (data);
+
+  if ((header & 0xffe00000) != 0xffe00000)
+    goto not_parsed;
+
+  /* thanks go to mp3parse */
+  if (header & (1 << 20)) {
+    lsf = (header & (1 << 19)) ? 0 : 1;
+    mpg25 = 0;
+  } else {
+    lsf = 1;
+    mpg25 = 1;
+  }
+
+  version = 1 + lsf + mpg25;
+  layer = 4 - ((header >> 17) & 0x3);
+
+  /* see http://www.codeproject.com/audio/MPEGAudioInfo.asp */
+  if (layer == 1)
+    spf = 384;
+  else if (layer == 2)
+    spf = 1152;
+  else if (version == 1) {
+    spf = 1152;
+  } else {
+    /* MPEG-2 or "2.5" */
+    spf = 576;
+  }
+
+  if (G_UNLIKELY (avipad->hdr.scale <= 1))
+    avipad->hdr.scale = spf;
+  else if (G_UNLIKELY (avipad->hdr.scale != spf)) {
+    GST_WARNING_OBJECT (avimux, "input mpeg audio has varying frame size");
+    goto cbr_fallback;
+  }
+
+  return GST_FLOW_OK;
+
+  /* EXITS */
+not_parsed:
+  {
+    GST_WARNING_OBJECT (avimux, "input mpeg audio is not parsed");
+    /* fall-through */
+  }
+cbr_fallback:
+  {
+    GST_WARNING_OBJECT (avimux, "falling back to CBR muxing");
+    avipad->hdr.scale = 1;
+    /* no need to check further */
+    avipad->hook = NULL;
+    return GST_FLOW_OK;
+  }
+}
+
+static void
+gst_avi_mux_audsink_set_fields (GstAviMux * avimux, GstAviAudioPad * avipad)
+{
+  if (avipad->parent.hdr.scale > 1) {
+    /* vbr case: fixed duration per frame/chunk */
+    avipad->parent.hdr.rate = avipad->auds.rate;
+    avipad->parent.hdr.samplesize = 0;
+    /* FIXME ?? some rumours say this should be largest audio chunk size */
+    avipad->auds.blockalign = avipad->parent.hdr.scale;
+  } else {
+    /* by spec, hdr.rate is av_bps related, is calculated that way in stop_file,
+     * and reduces to sample rate in PCM like cases */
+    avipad->parent.hdr.rate = avipad->auds.av_bps / avipad->auds.blockalign;
+    avipad->parent.hdr.samplesize = avipad->auds.blockalign;
+    avipad->parent.hdr.scale = 1;
+  }
+}
+
 static gboolean
 gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
 {
@@ -723,8 +819,10 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
       switch (mpegversion) {
         case 1:{
           gint layer = 3;
+          gboolean parsed = FALSE;
 
           gst_structure_get_int (structure, "layer", &layer);
+          gst_structure_get_boolean (structure, "parsed", &parsed);
           switch (layer) {
             case 3:
               avipad->auds.format = GST_RIFF_WAVE_FORMAT_MPEGL3;
@@ -734,12 +832,32 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
               avipad->auds.format = GST_RIFF_WAVE_FORMAT_MPEGL12;
               break;
           }
+          if (parsed) {
+            /* treat as VBR, should also cover CBR case;
+             * setup hook to parse frame header and determine spf */
+            avipad->parent.hook = gst_avi_mux_audsink_scan_mpeg_audio;
+          } else {
+            GST_WARNING_OBJECT (avimux, "unparsed MPEG audio input (?), "
+                "doing CBR muxing");
+          }
           break;
         }
         case 4:
-          GST_WARNING ("AAC");
+        {
+          GstBuffer *codec_data_buf = avipad->auds_codec_data;
+          guint codec;
+
+          /* vbr case needs some special handling */
+          if (!codec_data_buf || GST_BUFFER_SIZE (codec_data_buf) < 2) {
+            GST_WARNING_OBJECT (avimux, "no (valid) codec_data for AAC audio");
+            break;
+          }
           avipad->auds.format = GST_RIFF_WAVE_FORMAT_AAC;
+          /* need to determine frame length */
+          codec = GST_READ_UINT16_BE (GST_BUFFER_DATA (codec_data_buf));
+          avipad->parent.hdr.scale = (codec & 0x4) ? 960 : 1024;
           break;
+        }
       }
     } else if (!strcmp (mimetype, "audio/x-vorbis")) {
       avipad->auds.format = GST_RIFF_WAVE_FORMAT_VORBIS3;
@@ -787,11 +905,7 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
   if (!avipad->auds.format)
     goto refuse_caps;
 
-  /* by spec, hdr.rate is av_bps related, is calculated that way in stop_file,
-   * and reduces to sample rate in PCM like cases */
-  avipad->parent.hdr.rate = avipad->auds.av_bps / avipad->auds.blockalign;
-  avipad->parent.hdr.samplesize = avipad->auds.blockalign;
-  avipad->parent.hdr.scale = 1;
+  gst_avi_mux_audsink_set_fields (avimux, avipad);
 
   gst_object_unref (avimux);
   return TRUE;
@@ -1285,8 +1399,16 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
     GST_BUFFER_SIZE (buffer) += 12;
     buffdata = GST_BUFFER_DATA (buffer) + highmark;
 
-    /* update list size */
-    GST_WRITE_UINT32_LE (ptr, highmark - startsize - 4);
+    if (highmark - startsize - 4 == 4) {
+      /* no tags writen, remove the empty INFO LIST as it is useless
+       * and prevents playback in vlc */
+      highmark -= 12;
+      buffdata = GST_BUFFER_DATA (buffer) + highmark;
+      /* no need to erase the writen data, it will be overwriten anyway */
+    } else {
+      /* update list size */
+      GST_WRITE_UINT32_LE (ptr, highmark - startsize - 4);
+    }
   }
 
   /* avi data header */
@@ -1661,10 +1783,11 @@ gst_avi_mux_stop_file (GstAviMux * avimux)
               (NULL));
           audpad->auds.av_bps = 0;
         }
-        avipad->hdr.rate = audpad->auds.av_bps * avipad->hdr.scale;
       }
+      gst_avi_mux_audsink_set_fields (avimux, audpad);
       avimux->avi_hdr.max_bps += audpad->auds.av_bps;
-      avipad->hdr.length = (audpad->audio_time * avipad->hdr.rate) / GST_SECOND;
+      avipad->hdr.length = gst_util_uint64_scale (audpad->audio_time,
+          avipad->hdr.rate, avipad->hdr.scale * GST_SECOND);
     } else {
       GstAviVideoPad *vidpad = (GstAviVideoPad *) avipad;
 
@@ -1773,7 +1896,7 @@ gst_avi_mux_do_buffer (GstAviMux * avimux, GstAviPad * avipad)
 
   /* need to restart or start a next avix chunk ? */
   if ((avimux->is_bigfile ? avimux->datax_size : avimux->data_size) +
-      GST_BUFFER_SIZE (data) > 1024 * 1024 * 2000) {
+      GST_BUFFER_SIZE (data) > GST_AVI_MAX_SIZE) {
     if (avimux->enable_large_avi) {
       if ((res = gst_avi_mux_bigfile (avimux, FALSE)) != GST_FLOW_OK)
         return res;
@@ -1796,7 +1919,14 @@ gst_avi_mux_do_buffer (GstAviMux * avimux, GstAviPad * avipad)
     avimux->data_size += total_size;
   }
 
+  if (G_UNLIKELY (avipad->hook))
+    avipad->hook (avimux, avipad, data);
+
   if (avipad->is_video) {
+    /* the suggested buffer size is the max frame size */
+    if (avipad->hdr.bufsize < GST_BUFFER_SIZE (data))
+      avipad->hdr.bufsize = GST_BUFFER_SIZE (data);
+
     avimux->total_frames++;
 
     if (avimux->is_bigfile) {
