@@ -762,13 +762,15 @@ ifs_vfx_free (VisualFX * _this)
   free (data);
 }
 
-void
-ifs_visualfx_create (VisualFX * vfx)
+VisualFX
+ifs_visualfx_create (void)
 {
+  VisualFX vfx;
 
-  vfx->init = ifs_vfx_init;
-  vfx->free = ifs_vfx_free;
-  vfx->apply = ifs_vfx_apply;
-  vfx->fx_data = NULL;
-  vfx->params = NULL;
+  vfx.init = ifs_vfx_init;
+  vfx.free = ifs_vfx_free;
+  vfx.apply = ifs_vfx_apply;
+  vfx.fx_data = NULL;
+  vfx.params = NULL;
+  return vfx;
 }
