@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,10 +32,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpamrpay_debug);
 
 /* references:
  *
- * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File 
- *    Storage Format for the Adaptive Multi-Rate (AMR) and Adaptive 
+ * RFC 3267 - Real-Time Transport Protocol (RTP) Payload Format and File
+ *    Storage Format for the Adaptive Multi-Rate (AMR) and Adaptive
  *    Multi-Rate Wideband (AMR-WB) Audio Codecs.
- *    
+ *
  * ETSI TS 126 201 V6.0.0 (2004-12) - Digital cellular telecommunications system (Phase 2+);
  *                 Universal Mobile Telecommunications System (UMTS);
  *                          AMR speech codec, wideband;
@@ -45,10 +45,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpamrpay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_amrpay_details =
-GST_ELEMENT_DETAILS ("RTP packet payloader",
+GST_ELEMENT_DETAILS ("RTP AMR payloader",
     "Codec/Payloader/Network",
     "Payload-encode AMR or AMR-WB audio into RTP packets (RFC 3267)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 static GstStaticPadTemplate gst_rtp_amr_pay_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -116,12 +116,8 @@ gst_rtp_amr_pay_base_init (gpointer klass)
 static void
 gst_rtp_amr_pay_class_init (GstRtpAMRPayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -168,11 +164,11 @@ gst_rtp_amr_pay_setcaps (GstBaseRTPPayload * basepayload, GstCaps * caps)
 
   gst_basertppayload_set_outcaps (basepayload,
       "encoding-params", G_TYPE_STRING, "1", "octet-align", G_TYPE_STRING, "1",
-      /* don't set the defaults 
-       * 
+      /* don't set the defaults
+       *
        * "crc", G_TYPE_STRING, "0",
        * "robust-sorting", G_TYPE_STRING, "0",
-       * "interleaving", G_TYPE_STRING, "0", 
+       * "interleaving", G_TYPE_STRING, "0",
        */
       NULL);
 
@@ -192,6 +188,7 @@ static gint nb_frame_size[16] = {
   12, 13, 15, 17, 19, 20, 26, 31,
   5, -1, -1, -1, -1, -1, -1, 0
 };
+
 static gint wb_frame_size[16] = {
   17, 23, 32, 36, 40, 46, 50, 58,
   60, -1, -1, -1, -1, -1, -1, 0
@@ -228,7 +225,7 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
 
   GST_DEBUG_OBJECT (basepayload, "got %d bytes", size);
 
-  /* FIXME, only 
+  /* FIXME, only
    * octet aligned, no interleaving, single channel, no CRC,
    * no robust-sorting. To fix this you need to implement the downstream
    * negotiation function. */
@@ -267,27 +264,26 @@ gst_rtp_amr_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   /* now alloc output buffer */
   outbuf = gst_rtp_buffer_new_allocate (payload_len, 0, 0);
 
-  /* copy timestamp, or fabricate one */
-  if (timestamp != GST_CLOCK_TIME_NONE)
-    GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
-  else {
-    /* AMR (nb) and AMR-WB both have 20 ms per frame */
-    /* FIXME: when we do more than one AMR frame per packet, fix this */
-    gint count = basepayload->seqnum - basepayload->seqnum_base;
-
-    GST_BUFFER_TIMESTAMP (outbuf) = count * 20 * GST_MSECOND;
-  }
+  /* copy timestamp */
+  GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
 
+  /* FIXME: when we do more than one AMR frame per packet, fix this */
   if (duration != GST_CLOCK_TIME_NONE)
     GST_BUFFER_DURATION (outbuf) = duration;
   else {
     GST_BUFFER_DURATION (outbuf) = 20 * GST_MSECOND;
   }
 
+  if (GST_BUFFER_IS_DISCONT (buffer)) {
+    GST_DEBUG_OBJECT (basepayload, "discont, setting marker bit");
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
+    gst_rtp_buffer_set_marker (outbuf, TRUE);
+  }
+
   /* get payload, this is now writable */
   payload = gst_rtp_buffer_get_payload (outbuf);
 
-  /*   0 1 2 3 4 5 6 7 
+  /*   0 1 2 3 4 5 6 7
    *  +-+-+-+-+-+-+-+-+
    *  |  CMR  |R|R|R|R|
    *  +-+-+-+-+-+-+-+-+
