@@ -26,6 +26,10 @@
 #include <string.h>
 #include "gstrtpg729depay.h"
 
+GST_DEBUG_CATEGORY_STATIC (rtpg729depay_debug);
+#define GST_CAT_DEFAULT (rtpg729depay_debug)
+
+
 /* references:
  *
  * RFC 3551 (4.5.6)
@@ -93,6 +97,9 @@ gst_rtp_g729_depay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_g729_depay_sink_template));
 
   gst_element_class_set_details (element_class, &gst_rtp_g729depay_details);
+
+  GST_DEBUG_CATEGORY_INIT (rtpg729depay_debug, "rtpg729depay", 0,
+      "G.729 RTP Depayloader");
 }
 
 static void
@@ -192,10 +199,10 @@ gst_rtp_g729_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     goto bad_packet;
   }
 
-  GST_DEBUG_OBJECT (rtpg729depay, "payload len %d", payload_len);
+  GST_LOG_OBJECT (rtpg729depay, "payload len %d", payload_len);
 
   if ((payload_len % 10) == 2) {
-    GST_DEBUG_OBJECT (rtpg729depay, "G729 payload contains CNG frame");
+    GST_LOG_OBJECT (rtpg729depay, "G729 payload contains CNG frame");
   }
 
   outbuf = gst_rtp_buffer_get_payload_buffer (buf);
@@ -206,7 +213,7 @@ gst_rtp_g729_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
   }
 
-  GST_DEBUG ("gst_rtp_g729_depay_chain: pushing buffer of size %d",
+  GST_LOG_OBJECT (depayload, "pushing buffer of size %d",
       GST_BUFFER_SIZE (outbuf));
 
   return outbuf;
