@@ -70,6 +70,18 @@
 GST_DEBUG_CATEGORY (rtpjitterbuffer_debug);
 #define GST_CAT_DEFAULT (rtpjitterbuffer_debug)
 
+/* low and high threshold tell the queue when to start and stop buffering */
+#define LOW_THRESHOLD 0.2
+#define HIGH_THRESHOLD 0.8
+
+/* elementfactory information */
+static const GstElementDetails gst_rtp_jitter_buffer_details =
+GST_ELEMENT_DETAILS ("RTP packet jitter-buffer",
+    "Filter/Network/RTP",
+    "A buffer that deals with network jitter and other transmission faults",
+    "Philippe Kalaf <philippe.kalaf@collabora.co.uk>, "
+    "Wim Taymans <wim.taymans@gmail.com>");
+
 /* RTPJitterBuffer signals and args */
 enum
 {
@@ -77,7 +89,6 @@ enum
   SIGNAL_CLEAR_PT_MAP,
   SIGNAL_HANDLE_SYNC,
   SIGNAL_ON_NPT_STOP,
-  SIGNAL_SET_ACTIVE,
   LAST_SIGNAL
 };
 
@@ -85,8 +96,6 @@ enum
 #define DEFAULT_DROP_ON_LATENCY FALSE
 #define DEFAULT_TS_OFFSET       0
 #define DEFAULT_DO_LOST         FALSE
-#define DEFAULT_MODE            RTP_JITTER_BUFFER_MODE_SLAVE
-#define DEFAULT_PERCENT         0
 
 enum
 {
@@ -95,8 +104,6 @@ enum
   PROP_DROP_ON_LATENCY,
   PROP_TS_OFFSET,
   PROP_DO_LOST,
-  PROP_MODE,
-  PROP_PERCENT,
   PROP_LAST
 };
 
@@ -129,8 +136,6 @@ struct _GstRtpJitterBufferPrivate
   GCond *jbuf_cond;
   gboolean waiting;
   gboolean discont;
-  gboolean active;
-  guint64 out_offset;
 
   /* properties */
   guint latency_ms;
@@ -235,7 +240,6 @@ static GstPad *gst_rtp_jitter_buffer_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
 static void gst_rtp_jitter_buffer_release_pad (GstElement * element,
     GstPad * pad);
-static GstClock *gst_rtp_jitter_buffer_provide_clock (GstElement * element);
 
 /* pad overrides */
 static GstCaps *gst_rtp_jitter_buffer_getcaps (GstPad * pad);
@@ -263,27 +267,20 @@ static gboolean gst_rtp_jitter_buffer_query (GstPad * pad, GstQuery * query);
 
 static void
 gst_rtp_jitter_buffer_clear_pt_map (GstRtpJitterBuffer * jitterbuffer);
-static GstClockTime
-gst_rtp_jitter_buffer_set_active (GstRtpJitterBuffer * jitterbuffer,
-    gboolean active, guint64 base_time);
 
 static void
 gst_rtp_jitter_buffer_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jitter_buffer_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jitter_buffer_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jitter_buffer_sink_rtcp_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jitter_buffer_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jitter_buffer_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jitter_buffer_sink_rtcp_template));
 
-  gst_element_class_set_details_simple (element_class,
-      "RTP packet jitter-buffer", "Filter/Network/RTP",
-      "A buffer that deals with network jitter and other transmission faults",
-      "Philippe Kalaf <philippe.kalaf@collabora.co.uk>, "
-      "Wim Taymans <wim.taymans@gmail.com>");
+  gst_element_class_set_details (element_class, &gst_rtp_jitter_buffer_details);
 }
 
 static void
@@ -297,7 +294,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
 
   g_type_class_add_private (klass, sizeof (GstRtpJitterBufferPrivate));
 
-  gobject_class->finalize = gst_rtp_jitter_buffer_finalize;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_finalize);
 
   gobject_class->set_property = gst_rtp_jitter_buffer_set_property;
   gobject_class->get_property = gst_rtp_jitter_buffer_get_property;
@@ -311,7 +308,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
   g_object_class_install_property (gobject_class, PROP_LATENCY,
       g_param_spec_uint ("latency", "Buffer latency in ms",
           "Amount of ms to buffer", 0, G_MAXUINT, DEFAULT_LATENCY_MS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   /**
    * GstRtpJitterBuffer::drop-on-latency:
    *
@@ -321,7 +318,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
       g_param_spec_boolean ("drop-on-latency",
           "Drop buffers when maximum latency is reached",
           "Tells the jitterbuffer to never exceed the given latency in size",
-          DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE));
   /**
    * GstRtpJitterBuffer::ts-offset:
    *
@@ -344,27 +341,6 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
       g_param_spec_boolean ("do-lost", "Do Lost",
           "Send an event downstream when a packet is lost", DEFAULT_DO_LOST,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  /**
-   * GstRtpJitterBuffer::mode:
-   *
-   * Control the buffering and timestamping mode used by the jitterbuffer.
-   */
-  g_object_class_install_property (gobject_class, PROP_MODE,
-      g_param_spec_enum ("mode", "Mode",
-          "Control the buffering algorithm in use", RTP_TYPE_JITTER_BUFFER_MODE,
-          DEFAULT_MODE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  /**
-   * GstRtpJitterBuffer::percent:
-   *
-   * The percent of the jitterbuffer that is filled.
-   *
-   * Since: 0.10.19
-   */
-  g_object_class_install_property (gobject_class, PROP_PERCENT,
-      g_param_spec_int ("percent", "percent",
-          "The buffer filled percent", 0, 100,
-          0, G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   /**
    * GstRtpJitterBuffer::request-pt-map:
    * @buffer: the object which received the signal
@@ -416,35 +392,14 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
       G_STRUCT_OFFSET (GstRtpJitterBufferClass, clear_pt_map), NULL, NULL,
       g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
 
-  /**
-   * GstRtpJitterBuffer::set-active:
-   * @buffer: the object which received the signal
-   *
-   * Start pushing out packets with the given base time. This signal is only
-   * useful in buffering mode.
-   *
-   * Returns: the time of the last pushed packet.
-   *
-   * Since: 0.10.19
-   */
-  gst_rtp_jitter_buffer_signals[SIGNAL_SET_ACTIVE] =
-      g_signal_new ("set-active", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-      G_STRUCT_OFFSET (GstRtpJitterBufferClass, set_active), NULL, NULL,
-      gst_rtp_bin_marshal_UINT64__BOOL_UINT64, G_TYPE_UINT64, 2, G_TYPE_BOOLEAN,
-      G_TYPE_UINT64);
-
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_change_state);
   gstelement_class->request_new_pad =
       GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_request_new_pad);
   gstelement_class->release_pad =
       GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_release_pad);
-  gstelement_class->provide_clock =
-      GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_provide_clock);
 
   klass->clear_pt_map = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_clear_pt_map);
-  klass->set_active = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_set_active);
 
   GST_DEBUG_CATEGORY_INIT
       (rtpjitterbuffer_debug, "gstrtpjitterbuffer", 0, "RTP Jitter Buffer");
@@ -468,12 +423,6 @@ gst_rtp_jitter_buffer_init (GstRtpJitterBuffer * jitterbuffer,
   priv->jbuf_lock = g_mutex_new ();
   priv->jbuf_cond = g_cond_new ();
 
-  /* reset skew detection initialy */
-  rtp_jitter_buffer_reset_skew (priv->jbuf);
-  rtp_jitter_buffer_set_delay (priv->jbuf, priv->latency_ns);
-  rtp_jitter_buffer_set_buffering (priv->jbuf, FALSE);
-  priv->active = TRUE;
-
   priv->srcpad =
       gst_pad_new_from_static_template (&gst_rtp_jitter_buffer_src_template,
       "src");
@@ -654,12 +603,6 @@ wrong_pad:
   }
 }
 
-static GstClock *
-gst_rtp_jitter_buffer_provide_clock (GstElement * element)
-{
-  return gst_system_clock_obtain ();
-}
-
 static void
 gst_rtp_jitter_buffer_clear_pt_map (GstRtpJitterBuffer * jitterbuffer)
 {
@@ -671,53 +614,9 @@ gst_rtp_jitter_buffer_clear_pt_map (GstRtpJitterBuffer * jitterbuffer)
 
   JBUF_LOCK (priv);
   priv->clock_rate = -1;
-  /* do not clear current content, but refresh state for new arrival */
-  GST_DEBUG_OBJECT (jitterbuffer, "reset jitterbuffer");
-  rtp_jitter_buffer_reset_skew (priv->jbuf);
-  priv->last_popped_seqnum = -1;
-  priv->next_seqnum = -1;
   JBUF_UNLOCK (priv);
 }
 
-static GstClockTime
-gst_rtp_jitter_buffer_set_active (GstRtpJitterBuffer * jbuf, gboolean active,
-    guint64 offset)
-{
-  GstRtpJitterBufferPrivate *priv;
-  GstClockTime last_out;
-  GstBuffer *head;
-
-  priv = jbuf->priv;
-
-  JBUF_LOCK (priv);
-  GST_DEBUG_OBJECT (jbuf, "setting active %d with offset %" GST_TIME_FORMAT,
-      active, GST_TIME_ARGS (offset));
-
-  if (active != priv->active) {
-    /* add the amount of time spent in paused to the output offset. All
-     * outgoing buffers will have this offset applied to their timestamps in
-     * order to make them arrive in time in the sink. */
-    priv->out_offset = offset;
-    GST_DEBUG_OBJECT (jbuf, "out offset %" GST_TIME_FORMAT,
-        GST_TIME_ARGS (priv->out_offset));
-    priv->active = active;
-    JBUF_SIGNAL (priv);
-  }
-  if (!active) {
-    rtp_jitter_buffer_set_buffering (priv->jbuf, TRUE);
-  }
-  if ((head = rtp_jitter_buffer_peek (priv->jbuf))) {
-    /* head buffer timestamp and offset gives our output time */
-    last_out = GST_BUFFER_TIMESTAMP (head) + priv->ts_offset;
-  } else {
-    /* use last known time when the buffer is empty */
-    last_out = priv->last_out_time;
-  }
-  JBUF_UNLOCK (priv);
-
-  return last_out;
-}
-
 static GstCaps *
 gst_rtp_jitter_buffer_getcaps (GstPad * pad)
 {
@@ -964,6 +863,8 @@ gst_rtp_jitter_buffer_change_state (GstElement * element,
       priv->last_pt = -1;
       /* block until we go to PLAYING */
       priv->blocked = TRUE;
+      /* reset skew detection initialy */
+      rtp_jitter_buffer_reset_skew (priv->jbuf);
       JBUF_UNLOCK (priv);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
@@ -1013,34 +914,11 @@ gst_rtp_jitter_buffer_src_event (GstPad * pad, GstEvent * event)
   GstRtpJitterBufferPrivate *priv;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (jitterbuffer == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
   priv = jitterbuffer->priv;
 
   GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_LATENCY:
-    {
-      GstClockTime latency;
-
-      gst_event_parse_latency (event, &latency);
-
-      JBUF_LOCK (priv);
-      /* adjust the overall buffer delay to the total pipeline latency in
-       * buffering mode because if downstream consumes too fast (because of
-       * large latency or queues, we would start rebuffering again. */
-      if (rtp_jitter_buffer_get_mode (priv->jbuf) ==
-          RTP_JITTER_BUFFER_MODE_BUFFER) {
-        rtp_jitter_buffer_set_delay (priv->jbuf, latency);
-      }
-      JBUF_UNLOCK (priv);
-
-      ret = gst_pad_push_event (priv->sinkpad, event);
-      break;
-    }
     default:
       ret = gst_pad_push_event (priv->sinkpad, event);
       break;
@@ -1058,10 +936,6 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
   GstRtpJitterBufferPrivate *priv;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (jitterbuffer == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
   priv = jitterbuffer->priv;
 
   GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
@@ -1112,7 +986,7 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
        * we are flushing */
       ret = priv->srcresult == GST_FLOW_OK;
       if (ret && !priv->eos) {
-        GST_INFO_OBJECT (jitterbuffer, "queuing EOS");
+        GST_DEBUG_OBJECT (jitterbuffer, "queuing EOS");
         priv->eos = TRUE;
         JBUF_SIGNAL (priv);
       } else if (priv->eos) {
@@ -1140,7 +1014,6 @@ newseg_wrong_format:
   {
     GST_DEBUG_OBJECT (jitterbuffer, "received non TIME newsegment");
     ret = FALSE;
-    gst_event_unref (event);
     goto done;
   }
 }
@@ -1149,8 +1022,10 @@ static gboolean
 gst_rtp_jitter_buffer_sink_rtcp_event (GstPad * pad, GstEvent * event)
 {
   GstRtpJitterBuffer *jitterbuffer;
+  GstRtpJitterBufferPrivate *priv;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
+  priv = jitterbuffer->priv;
 
   GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
 
@@ -1169,12 +1044,10 @@ gst_rtp_jitter_buffer_sink_rtcp_event (GstPad * pad, GstEvent * event)
 }
 
 /*
- * Must be called with JBUF_LOCK held, will release the LOCK when emiting the
- * signal. The function returns GST_FLOW_ERROR when a parsing error happened and
- * GST_FLOW_WRONG_STATE when the element is shutting down. On success
- * GST_FLOW_OK is returned.
+ * Must be called with JBUF_LOCK held
  */
-static GstFlowReturn
+
+static gboolean
 gst_rtp_jitter_buffer_get_clock_rate (GstRtpJitterBuffer * jitterbuffer,
     guint8 pt)
 {
@@ -1191,10 +1064,8 @@ gst_rtp_jitter_buffer_get_clock_rate (GstRtpJitterBuffer * jitterbuffer,
   g_value_init (&ret, GST_TYPE_CAPS);
   g_value_set_boxed (&ret, NULL);
 
-  JBUF_UNLOCK (jitterbuffer->priv);
   g_signal_emitv (args, gst_rtp_jitter_buffer_signals[SIGNAL_REQUEST_PT_MAP], 0,
       &ret);
-  JBUF_LOCK_CHECK (jitterbuffer->priv, out_flushing);
 
   g_value_unset (&args[0]);
   g_value_unset (&args[1]);
@@ -1204,57 +1075,17 @@ gst_rtp_jitter_buffer_get_clock_rate (GstRtpJitterBuffer * jitterbuffer,
     goto no_caps;
 
   res = gst_jitter_buffer_sink_parse_caps (jitterbuffer, caps);
+
   gst_caps_unref (caps);
 
-  if (G_UNLIKELY (!res))
-    goto parse_failed;
-
-  return GST_FLOW_OK;
+  return res;
 
   /* ERRORS */
 no_caps:
   {
     GST_DEBUG_OBJECT (jitterbuffer, "could not get caps");
-    return GST_FLOW_ERROR;
+    return FALSE;
   }
-out_flushing:
-  {
-    GST_DEBUG_OBJECT (jitterbuffer, "we are flushing");
-    return GST_FLOW_WRONG_STATE;
-  }
-parse_failed:
-  {
-    GST_DEBUG_OBJECT (jitterbuffer, "parse failed");
-    return GST_FLOW_ERROR;
-  }
-}
-
-/* call with jbuf lock held */
-static void
-check_buffering_percent (GstRtpJitterBuffer * jitterbuffer, gint * percent)
-{
-  GstRtpJitterBufferPrivate *priv = jitterbuffer->priv;
-
-  /* too short a stream, or too close to EOS will never really fill buffer */
-  if (*percent != -1 && priv->npt_stop != -1 &&
-      priv->npt_stop - priv->npt_start <=
-      rtp_jitter_buffer_get_delay (priv->jbuf)) {
-    GST_DEBUG_OBJECT (jitterbuffer, "short stream; faking full buffer");
-    rtp_jitter_buffer_set_buffering (priv->jbuf, FALSE);
-    *percent = 100;
-  }
-}
-
-static void
-post_buffering_percent (GstRtpJitterBuffer * jitterbuffer, gint percent)
-{
-  GstMessage *message;
-
-  /* Post a buffering message */
-  message = gst_message_new_buffering (GST_OBJECT_CAST (jitterbuffer), percent);
-  gst_message_set_buffering_stats (message, GST_BUFFERING_LIVE, -1, -1, -1);
-
-  gst_element_post_message (GST_ELEMENT_CAST (jitterbuffer), message);
 }
 
 static GstFlowReturn
@@ -1267,7 +1098,6 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
   GstClockTime timestamp;
   guint64 latency_ts;
   gboolean tail;
-  gint percent = -1;
   guint8 pt;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
@@ -1314,10 +1144,7 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
 
   if (G_UNLIKELY (priv->clock_rate == -1)) {
     /* no clock rate given on the caps, try to get one with the signal */
-    if (gst_rtp_jitter_buffer_get_clock_rate (jitterbuffer,
-            pt) == GST_FLOW_WRONG_STATE)
-      goto out_flushing;
-
+    gst_rtp_jitter_buffer_get_clock_rate (jitterbuffer, pt);
     if (G_UNLIKELY (priv->clock_rate == -1))
       goto no_clock_rate;
   }
@@ -1383,7 +1210,7 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
     if (G_UNLIKELY (rtp_jitter_buffer_get_ts_diff (priv->jbuf) >= latency_ts)) {
       GstBuffer *old_buf;
 
-      old_buf = rtp_jitter_buffer_pop (priv->jbuf, &percent);
+      old_buf = rtp_jitter_buffer_pop (priv->jbuf);
 
       GST_DEBUG_OBJECT (jitterbuffer, "Queue full, dropping old packet #%d",
           gst_rtp_buffer_get_seq (old_buf));
@@ -1400,7 +1227,7 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
    * FALSE if a packet with the same seqnum was already in the queue, meaning we
    * have a duplicate. */
   if (G_UNLIKELY (!rtp_jitter_buffer_insert (priv->jbuf, buffer, timestamp,
-              priv->clock_rate, &tail, &percent)))
+              priv->clock_rate, priv->latency_ns, &tail)))
     goto duplicate;
 
   /* signal addition of new buffer when the _loop is waiting. */
@@ -1419,14 +1246,9 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
   GST_DEBUG_OBJECT (jitterbuffer, "Pushed packet #%d, now %d packets, tail: %d",
       seqnum, rtp_jitter_buffer_num_packets (priv->jbuf), tail);
 
-  check_buffering_percent (jitterbuffer, &percent);
-
 finished:
   JBUF_UNLOCK (priv);
 
-  if (percent != -1)
-    post_buffering_percent (jitterbuffer, percent);
-
   gst_object_unref (jitterbuffer);
 
   return ret;
@@ -1490,10 +1312,8 @@ apply_offset (GstRtpJitterBuffer * jitterbuffer, GstClockTime timestamp)
   if (timestamp == -1)
     return -1;
 
-  /* apply the timestamp offset, this is used for inter stream sync */
+  /* apply the timestamp offset */
   timestamp += priv->ts_offset;
-  /* add the offset, this is used when buffering */
-  timestamp += priv->out_offset;
 
   return timestamp;
 }
@@ -1524,7 +1344,7 @@ eos_reached (GstClock * clock, GstClockTime time, GstClockID id,
 
   JBUF_LOCK_CHECK (priv, flushing);
   if (priv->waiting) {
-    GST_INFO_OBJECT (jitterbuffer, "got the NPT timeout");
+    GST_DEBUG_OBJECT (jitterbuffer, "got the NPT timeout");
     priv->reached_npt_stop = TRUE;
     JBUF_SIGNAL (priv);
   }
@@ -1540,34 +1360,6 @@ flushing:
   }
 }
 
-static GstClockTime
-compute_elapsed (GstRtpJitterBuffer * jitterbuffer, GstBuffer * outbuf)
-{
-  guint64 ext_time, elapsed;
-  guint32 rtp_time;
-  GstRtpJitterBufferPrivate *priv;
-
-  priv = jitterbuffer->priv;
-  rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
-
-  GST_LOG_OBJECT (jitterbuffer, "rtp %" G_GUINT32_FORMAT ", ext %"
-      G_GUINT64_FORMAT, rtp_time, priv->ext_timestamp);
-
-  if (rtp_time < priv->ext_timestamp) {
-    ext_time = priv->ext_timestamp;
-  } else {
-    ext_time = gst_rtp_buffer_ext_timestamp (&priv->ext_timestamp, rtp_time);
-  }
-
-  if (ext_time > priv->clock_base)
-    elapsed = ext_time - priv->clock_base;
-  else
-    elapsed = 0;
-
-  elapsed = gst_util_uint64_scale_int (elapsed, GST_SECOND, priv->clock_rate);
-  return elapsed;
-}
-
 /*
  * This funcion will push out buffers on the source pad.
  *
@@ -1589,7 +1381,6 @@ gst_rtp_jitter_buffer_loop (GstRtpJitterBuffer * jitterbuffer)
   GstClock *clock;
   GstClockID id;
   GstClockTime sync_time;
-  gint percent = -1;
 
   priv = jitterbuffer->priv;
 
@@ -1600,39 +1391,6 @@ again:
     id = NULL;
     /* always wait if we are blocked */
     if (G_LIKELY (!priv->blocked)) {
-      /* we're buffering but not EOS, wait. */
-      if (!priv->eos && (!priv->active
-              || rtp_jitter_buffer_is_buffering (priv->jbuf))) {
-        GstClockTime elapsed, delay, left;
-
-        if (priv->estimated_eos == -1)
-          goto do_wait;
-
-        outbuf = rtp_jitter_buffer_peek (priv->jbuf);
-        if (outbuf != NULL) {
-          elapsed = compute_elapsed (jitterbuffer, outbuf);
-          if (GST_BUFFER_DURATION_IS_VALID (outbuf))
-            elapsed += GST_BUFFER_DURATION (outbuf);
-        } else {
-          GST_INFO_OBJECT (jitterbuffer, "no buffer, using last_elapsed");
-          elapsed = priv->last_elapsed;
-        }
-
-        delay = rtp_jitter_buffer_get_delay (priv->jbuf);
-
-        if (priv->estimated_eos > elapsed)
-          left = priv->estimated_eos - elapsed;
-        else
-          left = 0;
-
-        GST_INFO_OBJECT (jitterbuffer, "buffering, elapsed %" GST_TIME_FORMAT
-            " estimated_eos %" GST_TIME_FORMAT " left %" GST_TIME_FORMAT
-            " delay %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (elapsed), GST_TIME_ARGS (priv->estimated_eos),
-            GST_TIME_ARGS (left), GST_TIME_ARGS (delay));
-        if (left > delay)
-          goto do_wait;
-      }
       /* if we have a packet, we can exit the loop and grab it */
       if (rtp_jitter_buffer_num_packets (priv->jbuf) > 0)
         break;
@@ -1647,7 +1405,7 @@ again:
         GST_OBJECT_LOCK (jitterbuffer);
         clock = GST_ELEMENT_CLOCK (jitterbuffer);
         if (clock) {
-          GST_INFO_OBJECT (jitterbuffer, "scheduling timeout");
+          GST_DEBUG_OBJECT (jitterbuffer, "scheduling timeout");
           id = gst_clock_new_single_shot_id (clock, sync_time);
           gst_clock_id_wait_async (id, (GstClockCallback) eos_reached,
               jitterbuffer);
@@ -1655,13 +1413,10 @@ again:
         GST_OBJECT_UNLOCK (jitterbuffer);
       }
     }
-  do_wait:
     /* now we wait */
-    GST_DEBUG_OBJECT (jitterbuffer, "waiting");
     priv->waiting = TRUE;
     JBUF_WAIT (priv);
     priv->waiting = FALSE;
-    GST_DEBUG_OBJECT (jitterbuffer, "waiting done");
 
     if (id) {
       /* unschedule any pending async notifications we might have */
@@ -1710,7 +1465,7 @@ again:
     if (G_UNLIKELY (gap < 0)) {
       GST_DEBUG_OBJECT (jitterbuffer, "Old packet #%d, next #%d dropping",
           seqnum, next_seqnum);
-      outbuf = rtp_jitter_buffer_pop (priv->jbuf, &percent);
+      outbuf = rtp_jitter_buffer_pop (priv->jbuf);
       gst_buffer_unref (outbuf);
       goto again;
     }
@@ -1815,7 +1570,7 @@ again:
       GstEvent *event;
 
       /* we had a gap and thus we lost a packet. Create an event for this.  */
-      GST_DEBUG_OBJECT (jitterbuffer, "Packet #%d lost", next_seqnum);
+      GST_WARNING_OBJECT (jitterbuffer, "Packet #%d lost", next_seqnum);
       priv->num_late++;
       discont = TRUE;
 
@@ -1849,9 +1604,7 @@ again:
 push_buffer:
 
   /* when we get here we are ready to pop and push the buffer */
-  outbuf = rtp_jitter_buffer_pop (priv->jbuf, &percent);
-
-  check_buffering_percent (jitterbuffer, &percent);
+  outbuf = rtp_jitter_buffer_pop (priv->jbuf);
 
   if (G_UNLIKELY (discont || priv->discont)) {
     /* set DISCONT flag when we missed a packet. We pushed the buffer writable
@@ -1865,30 +1618,39 @@ push_buffer:
 
   /* update the elapsed time when we need to check against the npt stop time. */
   if (priv->npt_stop != -1 && priv->ext_timestamp != -1
-      && priv->clock_base != -1 && priv->clock_rate > 0) {
-    guint64 elapsed, estimated;
+      && priv->clock_base != -1) {
+    guint64 ext_time, elapsed, estimated;
+    guint32 rtp_time;
 
-    elapsed = compute_elapsed (jitterbuffer, outbuf);
+    rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
 
-    if (elapsed > priv->last_elapsed || !priv->last_elapsed) {
+    GST_LOG_OBJECT (jitterbuffer, "rtp %" G_GUINT32_FORMAT ", ext %"
+        G_GUINT64_FORMAT, rtp_time, priv->ext_timestamp);
+
+    if (rtp_time < priv->ext_timestamp) {
+      ext_time = priv->ext_timestamp;
+    } else {
+      ext_time = gst_rtp_buffer_ext_timestamp (&priv->ext_timestamp, rtp_time);
+    }
+
+    if (ext_time > priv->clock_base)
+      elapsed = ext_time - priv->clock_base;
+    else
+      elapsed = 0;
+
+    elapsed = gst_util_uint64_scale_int (elapsed, GST_SECOND, priv->clock_rate);
+
+    if (elapsed > priv->last_elapsed) {
       guint64 left;
 
       priv->last_elapsed = elapsed;
 
       left = priv->npt_stop - priv->npt_start;
-      GST_LOG_OBJECT (jitterbuffer, "left %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (left));
 
       if (elapsed > 0)
         estimated = gst_util_uint64_scale (out_time, left, elapsed);
-      else {
-        /* if there is almost nothing left,
-         * we may never advance enough to end up in the above case */
-        if (left < GST_SECOND)
-          estimated = GST_SECOND;
-        else
-          estimated = -1;
-      }
+      else
+        estimated = -1;
 
       GST_LOG_OBJECT (jitterbuffer, "elapsed %" GST_TIME_FORMAT ", estimated %"
           GST_TIME_FORMAT, GST_TIME_ARGS (elapsed), GST_TIME_ARGS (estimated));
@@ -1904,9 +1666,6 @@ push_buffer:
   priv->next_seqnum = (seqnum + 1) & 0xffff;
   JBUF_UNLOCK (priv);
 
-  if (percent != -1)
-    post_buffering_percent (jitterbuffer, percent);
-
   /* push buffer */
   GST_DEBUG_OBJECT (jitterbuffer,
       "Pushing buffer %d, timestamp %" GST_TIME_FORMAT, seqnum,
@@ -1967,7 +1726,7 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   GstRtpJitterBuffer *jitterbuffer;
   GstRtpJitterBufferPrivate *priv;
   GstFlowReturn ret = GST_FLOW_OK;
-  guint64 base_rtptime, base_time;
+  guint64 base_rtptime, timestamp;
   guint32 clock_rate;
   guint64 last_rtptime;
   guint32 ssrc;
@@ -1975,7 +1734,6 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   guint64 ext_rtptime, diff;
   guint32 rtptime;
   gboolean drop = FALSE;
-  guint64 clock_base;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
 
@@ -2006,17 +1764,14 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   ext_rtptime = gst_rtp_buffer_ext_timestamp (&ext_rtptime, rtptime);
 
   /* get the last values from the jitterbuffer */
-  rtp_jitter_buffer_get_sync (priv->jbuf, &base_rtptime, &base_time,
+  rtp_jitter_buffer_get_sync (priv->jbuf, &base_rtptime, &timestamp,
       &clock_rate, &last_rtptime);
 
-  clock_base = priv->clock_base;
-
   GST_DEBUG_OBJECT (jitterbuffer, "ext SR %" G_GUINT64_FORMAT ", base %"
-      G_GUINT64_FORMAT ", clock-rate %" G_GUINT32_FORMAT
-      ", clock-base %" G_GUINT64_FORMAT,
-      ext_rtptime, base_rtptime, clock_rate, clock_base);
+      G_GUINT64_FORMAT ", clock-rate %" G_GUINT32_FORMAT,
+      ext_rtptime, base_rtptime, clock_rate);
 
-  if (base_rtptime == -1 || clock_rate == -1 || base_time == -1) {
+  if (base_rtptime == -1 || clock_rate == -1 || timestamp == -1) {
     GST_DEBUG_OBJECT (jitterbuffer, "dropping, no RTP values");
     drop = TRUE;
   } else {
@@ -2032,12 +1787,8 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
         diff = ext_rtptime - last_rtptime;
         /* if bigger than 1 second, we drop it */
         if (diff > clock_rate) {
-          GST_DEBUG_OBJECT (jitterbuffer, "too far ahead");
-          /* should drop this, but some RTSP servers end up with bogus
-           * way too ahead RTCP packet when repeated PAUSE/PLAY,
-           * so still trigger rptbin sync but invalidate RTCP data
-           * (sync might use other methods) */
-          ext_rtptime = -1;
+          GST_DEBUG_OBJECT (jitterbuffer, "dropping, too far ahead");
+          drop = TRUE;
         }
         GST_DEBUG_OBJECT (jitterbuffer, "ext last %" G_GUINT64_FORMAT ", diff %"
             G_GUINT64_FORMAT, last_rtptime, diff);
@@ -2051,9 +1802,8 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
 
     s = gst_structure_new ("application/x-rtp-sync",
         "base-rtptime", G_TYPE_UINT64, base_rtptime,
-        "base-time", G_TYPE_UINT64, base_time,
+        "base-time", G_TYPE_UINT64, timestamp,
         "clock-rate", G_TYPE_UINT, clock_rate,
-        "clock-base", G_TYPE_UINT64, clock_base,
         "sr-ext-rtptime", G_TYPE_UINT64, ext_rtptime,
         "sr-buffer", GST_TYPE_BUFFER, buffer, NULL);
 
@@ -2096,8 +1846,6 @@ gst_rtp_jitter_buffer_query (GstPad * pad, GstQuery * query)
   gboolean res = FALSE;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (jitterbuffer == NULL))
-    return FALSE;
   priv = jitterbuffer->priv;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -2137,35 +1885,6 @@ gst_rtp_jitter_buffer_query (GstPad * pad, GstQuery * query)
       }
       break;
     }
-    case GST_QUERY_POSITION:
-    {
-      GstClockTime start, last_out;
-      GstFormat fmt;
-
-      gst_query_parse_position (query, &fmt, NULL);
-      if (fmt != GST_FORMAT_TIME) {
-        res = gst_pad_query_default (pad, query);
-        break;
-      }
-
-      JBUF_LOCK (priv);
-      start = priv->npt_start;
-      last_out = priv->last_out_time;
-      JBUF_UNLOCK (priv);
-
-      GST_DEBUG_OBJECT (jitterbuffer, "npt start %" GST_TIME_FORMAT
-          ", last out %" GST_TIME_FORMAT, GST_TIME_ARGS (start),
-          GST_TIME_ARGS (last_out));
-
-      if (GST_CLOCK_TIME_IS_VALID (start) && GST_CLOCK_TIME_IS_VALID (last_out)) {
-        /* bring 0-based outgoing time to stream time */
-        gst_query_set_position (query, GST_FORMAT_TIME, start + last_out);
-        res = TRUE;
-      } else {
-        res = gst_pad_query_default (pad, query);
-      }
-      break;
-    }
     default:
       res = gst_pad_query_default (pad, query);
       break;
@@ -2197,7 +1916,6 @@ gst_rtp_jitter_buffer_set_property (GObject * object,
       old_latency = priv->latency_ms;
       priv->latency_ms = new_latency;
       priv->latency_ns = priv->latency_ms * GST_MSECOND;
-      rtp_jitter_buffer_set_delay (priv->jbuf, priv->latency_ns);
       JBUF_UNLOCK (priv);
 
       /* post message if latency changed, this will inform the parent pipeline
@@ -2229,11 +1947,6 @@ gst_rtp_jitter_buffer_set_property (GObject * object,
       priv->do_lost = g_value_get_boolean (value);
       JBUF_UNLOCK (priv);
       break;
-    case PROP_MODE:
-      JBUF_LOCK (priv);
-      rtp_jitter_buffer_set_mode (priv->jbuf, g_value_get_enum (value));
-      JBUF_UNLOCK (priv);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -2271,25 +1984,6 @@ gst_rtp_jitter_buffer_get_property (GObject * object,
       g_value_set_boolean (value, priv->do_lost);
       JBUF_UNLOCK (priv);
       break;
-    case PROP_MODE:
-      JBUF_LOCK (priv);
-      g_value_set_enum (value, rtp_jitter_buffer_get_mode (priv->jbuf));
-      JBUF_UNLOCK (priv);
-      break;
-    case PROP_PERCENT:
-    {
-      gint percent;
-
-      JBUF_LOCK (priv);
-      if (priv->srcresult != GST_FLOW_OK)
-        percent = 100;
-      else
-        percent = rtp_jitter_buffer_get_percent (priv->jbuf);
-
-      g_value_set_int (value, percent);
-      JBUF_UNLOCK (priv);
-      break;
-    }
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
