@@ -27,6 +27,9 @@
 #include <glib.h>
 
 #ifdef G_OS_WIN32
+/* ws2_32.dll has getaddrinfo and freeaddrinfo on Windows XP and later.
+ * minwg32 headers check WINVER before allowing the use of these */
+#define WINVER 0x0501
 #include <winsock2.h>
 #include <ws2tcpip.h>
 
@@ -36,9 +39,10 @@
 
 #else
 #include <sys/time.h>
-#include <netinet/in.h>
-#include <netdb.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <net/if.h>
+#include <netdb.h>
 #include <sys/wait.h>
 #include <arpa/inet.h>
 #include <unistd.h>
@@ -71,12 +75,16 @@ gboolean gst_udp_net_utils_win32_wsa_startup (GstObject * obj);
 
 #endif
 
+int gst_udp_get_sockaddr_length(struct sockaddr_storage *addr);
+
 int gst_udp_get_addr      (const char *hostname, int port, struct sockaddr_storage *addr);
 int gst_udp_is_multicast  (struct sockaddr_storage *addr);
 
-int gst_udp_set_loop_ttl  (int sockfd, gboolean loop, int ttl);
+int gst_udp_set_loop      (int sockfd, gboolean loop);
+int gst_udp_set_ttl       (int sockfd, int ttl, gboolean is_multicast);
 
-int gst_udp_join_group    (int sockfd, struct sockaddr_storage *addr);
+int gst_udp_join_group    (int sockfd, struct sockaddr_storage *addr,
+                           gchar *iface);
 int gst_udp_leave_group   (int sockfd, struct sockaddr_storage *addr);
 
 #endif /* __GST_UDP_NET_UTILS_H__*/
