@@ -27,6 +27,7 @@
 #include <glib/gprintf.h>
 #include <libsoup/soup-address.h>
 #include <libsoup/soup-message.h>
+#include <libsoup/soup-misc.h>
 #include <libsoup/soup-server.h>
 #include <libsoup/soup-auth-domain.h>
 #include <libsoup/soup-auth-domain-basic.h>
@@ -441,7 +442,8 @@ souphttpsrc_suite (void)
   TCase *tc_chain, *tc_internet;
 
   g_type_init ();
-  g_thread_init (NULL);
+  if (!g_thread_supported ())
+    g_thread_init (NULL);
 
   s = suite_create ("souphttpsrc");
   tc_chain = tcase_create ("general");
@@ -450,7 +452,6 @@ souphttpsrc_suite (void)
   suite_add_tcase (s, tc_chain);
   run_server (&http_port, &https_port);
   tcase_add_test (tc_chain, test_first_buffer_has_offset);
-  tcase_add_test (tc_chain, test_https);
   tcase_add_test (tc_chain, test_redirect_yes);
   tcase_add_test (tc_chain, test_redirect_no);
   tcase_add_test (tc_chain, test_not_found);
@@ -462,6 +463,8 @@ souphttpsrc_suite (void)
   tcase_add_test (tc_chain, test_good_user_digest_auth);
   tcase_add_test (tc_chain, test_bad_user_digest_auth);
   tcase_add_test (tc_chain, test_bad_password_digest_auth);
+  if (soup_ssl_supported)
+    tcase_add_test (tc_chain, test_https);
 
   suite_add_tcase (s, tc_internet);
   tcase_set_timeout (tc_internet, 250);
@@ -562,8 +565,8 @@ run_server (guint * http_port, guint * https_port)
   SoupServer *server, *ssl_server;
   guint port = SOUP_ADDRESS_ANY_PORT;
   guint ssl_port = SOUP_ADDRESS_ANY_PORT;
-  const char *ssl_cert_file = G_STRINGIFY (CHECKDATA_DIR) "/test-cert.pem";
-  const char *ssl_key_file = G_STRINGIFY (CHECKDATA_DIR) "/test-key.pem";
+  const char *ssl_cert_file = GST_TEST_FILES_PATH "/test-cert.pem";
+  const char *ssl_key_file = GST_TEST_FILES_PATH "/test-key.pem";
   static int server_running = 0;
 
   SoupAuthDomain *domain = NULL;
